package com.css.simulation.resource.server.app.impl;

import api.common.pojo.common.PageVO;
import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.enums.MultiSimulationStatusEnum;
import api.common.pojo.param.project.MultiSimulationProjectParam;
import api.common.pojo.po.project.MultiSimulationProjectPO;
import api.common.pojo.vo.project.MultiSimulationProjectDetailVO;
import api.common.pojo.vo.project.MultiSimulationProjectVO;
import api.common.pojo.vo.project.SimulationManualProjectVO;
import api.common.util.StringUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.css.simulation.resource.server.app.service.job_manage.MultiSimulationProjectService;
import com.css.simulation.resource.server.infra.db.mysql.mapper.MultiSimulationProjectMapper;
import com.css.simulation.resource.server.infra.util.PageUtil;
import com.github.pagehelper.PageInfo;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class MultiSimulationProjectServiceImpl implements MultiSimulationProjectService {

    @Resource
    private MultiSimulationProjectMapper multiSimulationProjectMapper;
    @Override
    @SneakyThrows
    public ResponseBodyVO<PageInfo<MultiSimulationProjectVO>> selectMultiSimulationProject(MultiSimulationProjectParam param){
        if (Objects.isNull(param.getPageNum())){
            param.setPageNum(1);
        }
        if (Objects.isNull(param.getPageSize())){
            param.setPageSize(10);
        }
        if (StringUtils.isBlank(param.getProjectName())){
            param.setProjectName(null);
        }
        if (StringUtils.isBlank(param.getProjectKey())){
            param.setProjectKey(null);
        }

        setPage(param.getPageNum(), param.getPageSize());
        List<MultiSimulationProjectVO> multiSimulationProjectVOS = multiSimulationProjectMapper.selectProjectList(param);
        PageInfo<MultiSimulationProjectVO> pageInfo = new PageInfo<>(multiSimulationProjectVOS);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, pageInfo);
    }
    @Override
    @SneakyThrows
    public ResponseBodyVO<MultiSimulationProjectDetailVO> addMultiSimulationProject(MultiSimulationProjectParam param){
        MultiSimulationProjectPO build = MultiSimulationProjectPO.builder()
            .id(StringUtil.getRandomUUID())
            .projectName(param.getProjectName()== null ? "": param.getProjectName())
            .projectMaxSeconds(param.getProjectMaxSeconds() == null ? 0: param.getProjectMaxSeconds())
            .projectKey(param.getProjectKey() == null ? "" : param.getProjectKey())
            .projectDescription(param.getProjectDescription() == null ? "": param.getProjectDescription())
            .projectStatus(MultiSimulationStatusEnum.INIT_STATUS.getProjectStatus())
            .build();
        int i = multiSimulationProjectMapper.addMultiSimulationProject(build);
        if (i > 0){
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, MultiSimulationProjectDetailVO.builder().projectId(build.getId()).build());
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SERVER_FAILURE);
    }

    private void setPage(Integer pageNum, Integer pageSize) {
        PageVO pageVO = new PageVO();
        pageVO.setCurrentPage(pageNum);
        pageVO.setPageSize(pageSize);
        PageUtil.setPageInfo(pageVO);
    }
}
